"use strict";

var _superagent = _interopRequireDefault(require("superagent"));

var _ajv = _interopRequireDefault(require("ajv"));

var _package = require("./package.json");

var _md = _interopRequireDefault(require("md5"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _slicedToArray(arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return _sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var SERVER_URL = 'https://api.airshiphq.com';
var IDENTIFY_ENDPOINT = "".concat(SERVER_URL, "/v1/identify");
var GATING_INFO_ENDPOINT = "".concat(SERVER_URL, "/v1/gating-info");
var PLATFORM = 'nodejs';
var VERSION = _package.version;
var SDK_VERSION = "".concat(PLATFORM, ":").concat(VERSION);
var CONTROL_TYPE_BOOLEAN = 'boolean';
var CONTROL_TYPE_MULTIVARIATE = 'multivariate';
var DISTRIBUTION_TYPE_RULE_BASED = 'R';
var DISTRIBUTION_TYPE_PERCENTAGE_BASED = 'P';
var OBJECT_ATTRIBUTE_TYPE_STRING = 'STRING';
var OBJECT_ATTRIBUTE_TYPE_INT = 'INT';
var OBJECT_ATTRIBUTE_TYPE_FLOAT = 'FLOAT';
var OBJECT_ATTRIBUTE_TYPE_BOOLEAN = 'BOOLEAN';
var OBJECT_ATTRIBUTE_TYPE_DATE = 'DATE';
var OBJECT_ATTRIBUTE_TYPE_DATETIME = 'DATETIME';
var RULE_OPERATOR_TYPE_IS = 'IS';
var RULE_OPERATOR_TYPE_IS_NOT = 'IS_NOT';
var RULE_OPERATOR_TYPE_IN = 'IN';
var RULE_OPERATOR_TYPE_NOT_IN = 'NOT_IN';
var RULE_OPERATOR_TYPE_LT = 'LT';
var RULE_OPERATOR_TYPE_LTE = 'LTE';
var RULE_OPERATOR_TYPE_GT = 'GT';
var RULE_OPERATOR_TYPE_GTE = 'GTE';
var RULE_OPERATOR_TYPE_FROM = 'FROM';
var RULE_OPERATOR_TYPE_UNTIL = 'UNTIL';
var RULE_OPERATOR_TYPE_AFTER = 'AFTER';
var RULE_OPERATOR_TYPE_BEFORE = 'BEFORE';
var SCHEMA = {
  "type": "object",
  "properties": {
    "type": {
      "type": "string",
      "pattern": "^([A-Z][a-zA-Z]*)+$",
      "maxLength": 50
    },
    "isGroup": {
      "type": "boolean"
    },
    "id": {
      "type": ["string", "integer"],
      "pattern": "^[ -~]+$",
      "maxLength": 250,
      "minLength": 1
    },
    "displayName": {
      "type": "string",
      "maxLength": 250,
      "minLength": 1
    },
    "isAnonymous": {
      "type": "boolean"
    },
    "attributes": {
      "type": "object",
      "patternProperties": {
        "^[a-zA-Z][a-zA-Z_]{0,48}[a-zA-Z]$": {
          "oneOf": [{
            "type": "string",
            "maxLength": 3000
          }, {
            "type": "boolean"
          }, {
            "type": "number"
          }]
        }
      },
      "maxProperties": 100,
      "additionalProperties": false
    },
    "group": {
      "type": ["object", "null"],
      "properties": {
        "type": {
          "type": "string",
          "pattern": "^([A-Z][a-zA-Z]*)+$",
          "maxLength": 50
        },
        "isGroup": {
          "type": "boolean",
          "enum": [true]
        },
        "id": {
          "type": ["string", "integer"],
          "pattern": "^[ -~]+$",
          "maxLength": 250,
          "minLength": 1
        },
        "displayName": {
          "type": "string",
          "maxLength": 250,
          "minLength": 1
        },
        "attributes": {
          "type": "object",
          "patternProperties": {
            "^[a-zA-Z][a-zA-Z_]{0,48}[a-zA-Z]$": {
              "oneOf": [{
                "type": "string",
                "maxLength": 3000
              }, {
                "type": "boolean"
              }, {
                "type": "number"
              }]
            }
          },
          "maxProperties": 100,
          "additionalProperties": false
        }
      },
      "required": ["id"],
      "additionalProperties": false
    }
  },
  "required": ["id"],
  "additionalProperties": false
};

var makeid = function makeid() {
  var text = '';
  var possible = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';

  for (var i = 0; i < 6; i++) {
    text += possible.charAt(Math.floor(Math.random() * possible.length));
  }

  return text;
};

var SDK_ID = makeid();

var getHashedValue = function getHashedValue(s) {
  return parseInt((0, _md.default)(s), 16) * 1.0 / 340282366920938463463374607431768211455;
};

var Airship = function Airship(options) {
  var _this = this;

  _classCallCheck(this, Airship);

  Object.defineProperty(this, "init", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value() {
      _this.ingestionThread = setInterval(_this.triggerIngestion, _this.ingestionInterval);

      var getGatingInfo = function getGatingInfo() {
        return _superagent.default.get("".concat(GATING_INFO_ENDPOINT, "/").concat(_this.envKey)).set('api-key', _this.apiKey).set('sdk-version', SDK_VERSION).timeout(_this.timeout);
      };

      var maybeGetGatingInfo = function maybeGetGatingInfo() {
        var gatingInfoPromise = getGatingInfo().then(function (res) {
          var gatingInfo = res.body;

          if (gatingInfo.serverInfo === 'maintenance') {
            console.error('Airship is currently going through maintenance');
          } else {
            var gatingInfoMap = _this._getGatingInfoMap(gatingInfo);

            _this.gatingInfo = gatingInfo;
            _this.gatingInfoMap = gatingInfoMap;

            if (gatingInfo.pollingInterval) {
              var newPollingInterval = gatingInfo.pollingInterval * 1000;

              if (_this.pollingInterval !== newPollingInterval) {
                clearInterval(_this.pollingThread);
                _this.pollingInterval = newPollingInterval;
                _this.pollingThread = setInterval(maybeGetGatingInfo, _this.pollingInterval);
              }
            }

            if (gatingInfo.ingestionInterval) {
              var newIngestionInterval = gatingInfo.ingestionInterval * 1000;

              if (_this.ingestionInterval !== newIngestionInterval) {
                clearInterval(_this.ingestionThread);
                _this.ingestionInterval = newIngestionInterval;
                _this.ingestionThread = setInterval(_this.triggerIngestion, _this.ingestionInterval);
              }
            }
          }
        }).catch(function (err) {
          console.error('Failed to connect to Airship server');
        });
        return gatingInfoPromise;
      };

      var initialGatingInfoPromise = maybeGetGatingInfo();
      _this.pollingThread = setInterval(maybeGetGatingInfo, _this.pollingInterval);
      return initialGatingInfoPromise;
    }
  });
  Object.defineProperty(this, "triggerIngestion", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value() {
      if (_this.ingestionBatch.length === 0) {
        return;
      }

      var doIngestion = function doIngestion() {
        var payload = _this.ingestionBatch;
        _this.ingestionBatch = [];
        return _superagent.default.post(IDENTIFY_ENDPOINT).type('application/json').set('api-key', _this.apiKey).timeout(_this.timeout).send({
          envKey: _this.envKey,
          objects: payload
        }).then(function () {}).catch(function () {});
      };

      return doIngestion();
    }
  });
  Object.defineProperty(this, "_getGatingInfoMap", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(gatingInfo) {
      var map = {};
      var controls = gatingInfo.controls;

      for (var i = 0; i < controls.length; i++) {
        var control = controls[i];
        var controlInfo = {};
        controlInfo.id = control.id;
        controlInfo.isOn = control.isOn;
        controlInfo.ruleBasedDistributionDefaultVariation = control.ruleBasedDistributionDefaultVariation;
        controlInfo.ruleSets = control.ruleSets;
        controlInfo.distributions = control.distributions;
        controlInfo.type = control.type;
        controlInfo.defaultVariation = control.defaultVariation;
        var enablements = control.enablements;
        var enablementsInfo = {};

        for (var j = 0; j < enablements.length; j++) {
          var enablement = enablements[j];
          var clientIdentitiesMap = enablementsInfo[enablement.clientObjectTypeName];

          if (clientIdentitiesMap === undefined) {
            enablementsInfo[enablement.clientObjectTypeName] = {};
          }

          enablementsInfo[enablement.clientObjectTypeName][enablement.clientObjectIdentity] = [enablement.isEnabled, enablement.variation];
        }

        controlInfo.enablementsInfo = enablementsInfo;
        map[control.shortName] = controlInfo;
      }

      return map;
    }
  });
  Object.defineProperty(this, "_ingestAsync", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(gateStats) {
      _this.ingestionBatch.push(gateStats);

      if (_this.firstIngestion) {
        setImmediate(_this.triggerIngestion);
        _this.firstIngestion = false;
        return;
      }

      if (_this.ingestionBatch.length >= _this.ingestionMaxBatchSize) {
        setImmediate(_this.triggerIngestion);
      }
    }
  });
  Object.defineProperty(this, "_satisfiesRule", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(rule, object) {
      var attributeType = rule.attributeType;
      var operator = rule.operator;
      var attributeName = rule.attributeName;
      var value = rule.value;
      var valueList = rule.valueList;

      if (object.attributes === undefined || object.attributes[attributeName] === undefined) {
        return false;
      }

      var attributeVal = object.attributes[attributeName];

      if (attributeType === OBJECT_ATTRIBUTE_TYPE_STRING) {
        if (operator === RULE_OPERATOR_TYPE_IS) {
          return attributeVal === value;
        } else if (operator === RULE_OPERATOR_TYPE_IS_NOT) {
          return attributeVal !== value;
        } else if (operator === RULE_OPERATOR_TYPE_IN) {
          return valueList.indexOf(attributeVal) >= 0;
        } else if (operator === RULE_OPERATOR_TYPE_NOT_IN) {
          return valueList.indexOf(attributeVal) === -1;
        } else {
          return false;
        }
      } else if (attributeType === OBJECT_ATTRIBUTE_TYPE_INT) {
        value = value && parseInt(value);
        valueList = valueList && valueList.map(function (v) {
          return parseInt(v);
        });

        if (operator === RULE_OPERATOR_TYPE_IS) {
          return attributeVal === value;
        } else if (operator === RULE_OPERATOR_TYPE_IS_NOT) {
          return attributeVal !== value;
        } else if (operator === RULE_OPERATOR_TYPE_IN) {
          return valueList.indexOf(attributeVal) >= 0;
        } else if (operator === RULE_OPERATOR_TYPE_NOT_IN) {
          return valueList.indexOf(attributeVal) === -1;
        } else if (operator === RULE_OPERATOR_TYPE_LT) {
          return attributeVal < value;
        } else if (operator === RULE_OPERATOR_TYPE_LTE) {
          return attributeVal <= value;
        } else if (operator === RULE_OPERATOR_TYPE_GT) {
          return attributeVal > value;
        } else if (operator === RULE_OPERATOR_TYPE_GTE) {
          return attributeVal >= value;
        } else {
          return false;
        }
      } else if (attributeType === OBJECT_ATTRIBUTE_TYPE_FLOAT) {
        value = value && parseFloat(value);
        valueList = valueList && valueList.map(function (v) {
          return parseFloat(v);
        });

        if (operator === RULE_OPERATOR_TYPE_IS) {
          return attributeVal === value;
        } else if (operator === RULE_OPERATOR_TYPE_IS_NOT) {
          return attributeVal !== value;
        } else if (operator === RULE_OPERATOR_TYPE_IN) {
          return valueList.indexOf(attributeVal) >= 0;
        } else if (operator === RULE_OPERATOR_TYPE_NOT_IN) {
          return valueList.indexOf(attributeVal) === -1;
        } else if (operator === RULE_OPERATOR_TYPE_LT) {
          return attributeVal < value;
        } else if (operator === RULE_OPERATOR_TYPE_LTE) {
          return attributeVal <= value;
        } else if (operator === RULE_OPERATOR_TYPE_GT) {
          return attributeVal > value;
        } else if (operator === RULE_OPERATOR_TYPE_GTE) {
          return attributeVal >= value;
        } else {
          return false;
        }
      } else if (attributeType === OBJECT_ATTRIBUTE_TYPE_BOOLEAN) {
        value = value === 'true';

        if (operator === RULE_OPERATOR_TYPE_IS) {
          return attributeVal === value;
        } else if (operator === RULE_OPERATOR_TYPE_IS_NOT) {
          return attributeVal !== value;
        } else {
          return false;
        }
      } else if (attributeType === OBJECT_ATTRIBUTE_TYPE_DATE) {
        var unixTimestamp = new Date(attributeVal).getTime();

        if (isNaN(unixTimestamp)) {
          return false;
        }

        var isoFormat = new Date(attributeVal).toISOString();

        if (!isoFormat.endsWith('T00:00:00.000Z')) {
          return false;
        }

        value = value && new Date(value).getTime();
        valueList = valueList && valueList.map(function (v) {
          return new Date(v).getTime();
        });
        attributeVal = unixTimestamp;

        if (operator === RULE_OPERATOR_TYPE_IS) {
          return attributeVal === value;
        } else if (operator === RULE_OPERATOR_TYPE_IS_NOT) {
          return attributeVal !== value;
        } else if (operator === RULE_OPERATOR_TYPE_IN) {
          return valueList.indexOf(attributeVal) >= 0;
        } else if (operator === RULE_OPERATOR_TYPE_NOT_IN) {
          return valueList.indexOf(attributeVal) === -1;
        } else if (operator === RULE_OPERATOR_TYPE_FROM) {
          return attributeVal >= value;
        } else if (operator === RULE_OPERATOR_TYPE_UNTIL) {
          return attributeVal <= value;
        } else if (operator === RULE_OPERATOR_TYPE_AFTER) {
          return attributeVal > value;
        } else if (operator === RULE_OPERATOR_TYPE_BEFORE) {
          return attributeVal < value;
        } else {
          return false;
        }
      } else if (attributeType === OBJECT_ATTRIBUTE_TYPE_DATETIME) {
        // .getTime() respects timezones
        var _unixTimestamp = new Date(attributeVal).getTime();

        if (isNaN(_unixTimestamp)) {
          return false;
        }

        value = value && new Date(value).getTime();
        valueList = valueList && valueList.map(function (v) {
          return new Date(v).getTime();
        });
        attributeVal = _unixTimestamp;

        if (operator === RULE_OPERATOR_TYPE_IS) {
          return attributeVal === value;
        } else if (operator === RULE_OPERATOR_TYPE_IS_NOT) {
          return attributeVal !== value;
        } else if (operator === RULE_OPERATOR_TYPE_IN) {
          return valueList.indexOf(attributeVal) >= 0;
        } else if (operator === RULE_OPERATOR_TYPE_NOT_IN) {
          return valueList.indexOf(attributeVal) === -1;
        } else if (operator === RULE_OPERATOR_TYPE_FROM) {
          return attributeVal >= value;
        } else if (operator === RULE_OPERATOR_TYPE_UNTIL) {
          return attributeVal <= value;
        } else if (operator === RULE_OPERATOR_TYPE_AFTER) {
          return attributeVal > value;
        } else if (operator === RULE_OPERATOR_TYPE_BEFORE) {
          return attributeVal < value;
        } else {
          return false;
        }
      } else {
        return false;
      }
    }
  });
  Object.defineProperty(this, "_getGateValuesForObject", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(controlInfo, object) {
      if (controlInfo.enablementsInfo[object.type] !== undefined) {
        if (controlInfo.enablementsInfo[object.type][object.id] !== undefined) {
          var _controlInfo$enableme = _slicedToArray(controlInfo.enablementsInfo[object.type][object.id], 2),
              isEnabled = _controlInfo$enableme[0],
              variation = _controlInfo$enableme[1];

          return {
            isEnabled: isEnabled,
            variation: variation,
            isEligible: isEnabled,
            _fromEnablement: true
          };
        }
      }

      var sampledInsideBasePopulation = false;
      var isEligible = false;

      for (var i = 0; i < controlInfo.ruleSets.length; i++) {
        if (sampledInsideBasePopulation) {
          break;
        }

        var ruleSet = controlInfo.ruleSets[i];
        var rules = ruleSet.rules;

        if (ruleSet.clientObjectTypeName !== object.type) {
          continue;
        }

        var satisfiesAllRules = true;

        for (var j = 0; j < rules.length; j++) {
          var rule = rules[j];
          satisfiesAllRules = satisfiesAllRules && _this._satisfiesRule(rule, object);
        }

        if (satisfiesAllRules) {
          isEligible = true;
          var hashKey = "SAMPLING:control_".concat(controlInfo.id, ":env_").concat(_this.gatingInfo.env.id, ":rule_set_").concat(ruleSet.id, ":client_object_").concat(object.type, "_").concat(object.id);

          if (getHashedValue(hashKey) <= ruleSet.samplingPercentage) {
            sampledInsideBasePopulation = true;
          }
        }
      }

      if (!sampledInsideBasePopulation) {
        return {
          isEnabled: false,
          variation: null,
          isEligible: isEligible
        };
      }

      if (controlInfo.type === CONTROL_TYPE_BOOLEAN) {
        return {
          isEnabled: true,
          variation: null,
          isEligible: true
        };
      } else if (controlInfo.type === CONTROL_TYPE_MULTIVARIATE) {
        if (controlInfo.distributions.length === 0) {
          return {
            isEnabled: true,
            variation: controlInfo.defaultVariation,
            isEligible: true
          };
        }

        var percentageBasedDistributions = controlInfo.distributions.filter(function (d) {
          return d.type === DISTRIBUTION_TYPE_PERCENTAGE_BASED;
        });
        var ruleBasedDistributions = controlInfo.distributions.filter(function (d) {
          return d.type === DISTRIBUTION_TYPE_RULE_BASED;
        });

        if (percentageBasedDistributions.length !== 0 && ruleBasedDistributions.length !== 0) {
          console.error('Rule integrity error: please contact support@airshiphq.com');
          return {
            isEnabled: false,
            variation: null,
            isEligible: false
          };
        }

        if (percentageBasedDistributions.length !== 0) {
          var delta = 0.0001;
          var sum_percentages = 0.0;
          var running_percentages = [];

          for (var _i2 = 0; _i2 < percentageBasedDistributions.length; _i2++) {
            var distribution = percentageBasedDistributions[_i2];
            sum_percentages += distribution.percentage;

            if (running_percentages.length === 0) {
              running_percentages.push(distribution.percentage);
            } else {
              running_percentages.push(running_percentages[running_percentages.length - 1] + distribution.percentage);
            }
          }

          if (Math.abs(1.0 - sum_percentages) > delta) {
            console.error('Rule integrity error: please contact support@airshiphq.com');
            return {
              isEnabled: false,
              variation: null,
              isEligible: false
            };
          }

          var _hashKey = "DISTRIBUTION:control_".concat(controlInfo.id, ":env_").concat(_this.gatingInfo.env.id, ":client_object_").concat(object.type, "_").concat(object.id);

          var hashedPercentage = getHashedValue(_hashKey);

          for (var _i3 = 0; _i3 < running_percentages.length; _i3++) {
            var percentage = running_percentages[_i3];

            if (hashedPercentage <= percentage) {
              return {
                isEnabled: true,
                variation: percentageBasedDistributions[_i3].variation,
                isEligible: true
              };
            }
          }

          return {
            isEnabled: true,
            variation: percentageBasedDistributions[percentageBasedDistributions.length - 1].variation,
            isEligible: true
          };
        } else {
          for (var _i4 = 0; _i4 < ruleBasedDistributions.length; _i4++) {
            var _distribution = ruleBasedDistributions[_i4];
            var _ruleSet = _distribution.ruleSet;
            var _rules = _ruleSet.rules;

            if (_ruleSet.clientObjectTypeName !== object.type) {
              continue;
            }

            var _satisfiesAllRules = true;

            for (var _j = 0; _j < _rules.length; _j++) {
              var _rule = _rules[_j];
              _satisfiesAllRules = _satisfiesAllRules && _this._satisfiesRule(_rule, object);
            }

            if (_satisfiesAllRules) {
              return {
                isEnabled: true,
                variation: _distribution.variation,
                isEligible: true
              };
            }
          }

          return {
            isEnabled: true,
            variation: controlInfo.ruleBasedDistributionDefaultVariation || controlInfo.defaultVariation,
            isEligible: true,
            _ruleBasedDefaultVariation: true
          };
        }
      } else {
        return {
          isEnabled: false,
          variation: null,
          isEligible: false
        };
      }
    }
  });
  Object.defineProperty(this, "_getGateValues", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(controlShortName, object) {
      if (_this.gatingInfoMap[controlShortName] === undefined) {
        return {
          isEnabled: false,
          variation: null,
          isEligible: false,
          _shouldSendStats: false
        };
      }

      var controlInfo = _this.gatingInfoMap[controlShortName];

      if (!controlInfo.isOn) {
        return {
          isEnabled: false,
          variation: null,
          isEligible: false,
          _shouldSendStats: true
        };
      }

      var group = null;

      if (object.group !== undefined) {
        group = object.group;
      }

      var result = _this._getGateValuesForObject(controlInfo, object);

      if (group !== null) {
        if (group.type === undefined) {
          group.type = "".concat(object.type, "Group");
          group.isGroup = true;
        }

        var groupResult = _this._getGateValuesForObject(controlInfo, group);

        if (result._fromEnablement && !result.isEnabled) {// Do nothing
        } else if (!result._fromEnablement && groupResult._fromEnablement && !groupResult.isEnabled) {
          result.isEnabled = groupResult.isEnabled;
          result.variation = groupResult.variation;
          result.isEligible = groupResult.isEligible;
        } else if (result.isEnabled) {
          if (result._ruleBasedDefaultVariation) {
            if (groupResult.isEnabled) {
              result.isEnabled = groupResult.isEnabled;
              result.variation = groupResult.variation;
              result.isEligible = groupResult.isEligible;
            } else {// Do nothing
            }
          } else {// Do nothing
            }
        } else if (groupResult.isEnabled) {
          result.isEnabled = groupResult.isEnabled;
          result.variation = groupResult.variation;
          result.isEligible = groupResult.isEligible;
        } else {// Do nothing
        }
      }

      result._shouldSendStats = true;
      return result;
    }
  });
  Object.defineProperty(this, "_cloneObject", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(object) {
      var clone = Object.assign({}, object);

      if (object.attributes !== undefined) {
        clone.attributes = Object.assign({}, object.attributes);
      }

      if (object.group !== undefined) {
        clone.group = Object.assign({}, object.group);

        if (object.group.attributes !== undefined) {
          clone.group.attributes = Object.assign({}, object.group.attributes);
        }
      }

      return clone;
    }
  });
  Object.defineProperty(this, "_validateNesting", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(object) {
      if (object.isGroup === true && object.group !== undefined) {
        return 'A group cannot be nested inside another group';
      }

      return null;
    }
  });
  Object.defineProperty(this, "_validateAnonymous", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(object) {
      if (object.isAnonymous === true && object.group !== undefined) {
        return 'An anonymous object cannot belong to a group';
      }

      if (object.isGroup === true && object.isAnonymous === true) {
        return 'A group cannot be anonymous';
      }

      return null;
    }
  });
  Object.defineProperty(this, "_maybeTransformId", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(object) {
      if (Number.isInteger(object.id)) {
        var idStr = '' + object.id;

        if (idStr.length > 250) {
          return 'Integer id must have 250 digits or less';
        }

        object.id = idStr;
      }

      var group = null;

      if (object.group !== undefined) {
        group = object.group;
      }

      if (group !== null) {
        if (Number.isInteger(group.id)) {
          var _idStr = '' + group.id;

          if (_idStr.length > 250) {
            return 'Integer id must have 250 digits or less';
          }

          group.id = _idStr;
        }
      }

      return null;
    }
  });
  Object.defineProperty(this, "_enrichWithMetadata", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(controlShortName, stats) {
      var controlInfo = _this.gatingInfoMap[controlShortName];

      if (controlInfo !== undefined) {
        stats.sdkGateControlId = controlInfo.id;
      }

      stats.sdkEnvId = _this.gatingInfo['env'].id;
    }
  });
  Object.defineProperty(this, "_maybeAddFields", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(object) {
      if (object.type === undefined) {
        object.type = 'User';
      }

      if (object.displayName === undefined) {
        object.displayName = '' + object.id;
      }

      var group = null;

      if (object.group !== undefined) {
        group = object.group;
      }

      if (group !== null && group.displayName === undefined) {
        group.displayName = '' + group.id;
      }
    }
  });
  Object.defineProperty(this, "isEnabled", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(controlShortName, object) {
      var defaultValue = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      var valid = _this.validate(object);

      if (!valid) {
        console.error(_this.validate.errors);
        return defaultValue;
      }

      object = _this._cloneObject(object);

      _this._maybeAddFields(object);

      var error = _this._validateNesting(object) || _this._validateAnonymous(object) || _this._maybeTransformId(object);

      if (error) {
        console.error(error);
        return defaultValue;
      }

      if (_this.gatingInfoMap === null) {
        return defaultValue;
      }

      var gateTimestamp = new Date().toISOString();
      var start = process.hrtime();

      var _this$_getGateValues = _this._getGateValues(controlShortName, object),
          isEnabled = _this$_getGateValues.isEnabled,
          variation = _this$_getGateValues.variation,
          isEligible = _this$_getGateValues.isEligible,
          _shouldSendStats = _this$_getGateValues._shouldSendStats;

      var end = process.hrtime(start);

      if (_shouldSendStats) {
        var sdkGateTimestamp = gateTimestamp;
        var sdkGateLatency = "".concat(end[1] / 1000.0, "us");
        var sdkVersion = SDK_VERSION;
        var stats = {};
        stats.sdkGateControlShortName = controlShortName;
        stats.sdkGateTimestamp = sdkGateTimestamp;
        stats.sdkGateLatency = sdkGateLatency;
        stats.sdkGateValue = isEnabled;
        stats.sdkGateVariation = variation;
        stats.sdkGateEligibility = isEligible;
        stats.sdkGateType = 'value';

        _this._enrichWithMetadata(controlShortName, stats);

        stats.sdkVersion = sdkVersion;
        stats.sdkId = SDK_ID;
        object.stats = stats;

        _this._ingestAsync(object);
      }

      return isEnabled;
    }
  });
  Object.defineProperty(this, "getVariation", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(controlShortName, object) {
      var defaultValue = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;

      var valid = _this.validate(object);

      if (!valid) {
        console.error(_this.validate.errors);
        return defaultValue;
      }

      object = _this._cloneObject(object);

      _this._maybeAddFields(object);

      var error = _this._validateNesting(object) || _this._validateAnonymous(object) || _this._maybeTransformId(object);

      if (error) {
        console.error(error);
        return defaultValue;
      }

      if (_this.gatingInfoMap === null) {
        return defaultValue;
      }

      var gateTimestamp = new Date().toISOString();
      var start = process.hrtime();

      var _this$_getGateValues2 = _this._getGateValues(controlShortName, object),
          isEnabled = _this$_getGateValues2.isEnabled,
          variation = _this$_getGateValues2.variation,
          isEligible = _this$_getGateValues2.isEligible,
          _shouldSendStats = _this$_getGateValues2._shouldSendStats;

      var end = process.hrtime(start);

      if (_shouldSendStats) {
        var sdkGateTimestamp = gateTimestamp;
        var sdkGateLatency = "".concat(end[1] / 1000.0, "us");
        var sdkVersion = SDK_VERSION;
        var stats = {};
        stats.sdkGateControlShortName = controlShortName;
        stats.sdkGateTimestamp = sdkGateTimestamp;
        stats.sdkGateLatency = sdkGateLatency;
        stats.sdkGateValue = isEnabled;
        stats.sdkGateVariation = variation;
        stats.sdkGateEligibility = isEligible;
        stats.sdkGateType = 'variation';

        _this._enrichWithMetadata(controlShortName, stats);

        stats.sdkVersion = sdkVersion;
        stats.sdkId = SDK_ID;
        object.stats = stats;

        _this._ingestAsync(object);
      }

      return variation;
    }
  });
  Object.defineProperty(this, "isEligible", {
    configurable: true,
    enumerable: true,
    writable: true,
    value: function value(controlShortName, object) {
      var defaultValue = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      var valid = _this.validate(object);

      if (!valid) {
        console.error(_this.validate.errors);
        return defaultValue;
      }

      object = _this._cloneObject(object);

      _this._maybeAddFields(object);

      var error = _this._validateNesting(object) || _this._validateAnonymous(object) || _this._maybeTransformId(object);

      if (error) {
        console.error(error);
        return defaultValue;
      }

      if (_this.gatingInfoMap === null) {
        return defaultValue;
      }

      var gateTimestamp = new Date().toISOString();
      var start = process.hrtime();

      var _this$_getGateValues3 = _this._getGateValues(controlShortName, object),
          isEnabled = _this$_getGateValues3.isEnabled,
          variation = _this$_getGateValues3.variation,
          isEligible = _this$_getGateValues3.isEligible,
          _shouldSendStats = _this$_getGateValues3._shouldSendStats;

      var end = process.hrtime(start);

      if (_shouldSendStats) {
        var sdkGateTimestamp = gateTimestamp;
        var sdkGateLatency = "".concat(end[1] / 1000.0, "us");
        var sdkVersion = SDK_VERSION;
        var stats = {};
        stats.sdkGateControlShortName = controlShortName;
        stats.sdkGateTimestamp = sdkGateTimestamp;
        stats.sdkGateLatency = sdkGateLatency;
        stats.sdkGateValue = isEnabled;
        stats.sdkGateVariation = variation;
        stats.sdkGateEligibility = isEligible;
        stats.sdkGateType = 'eligibility';

        _this._enrichWithMetadata(controlShortName, stats);

        stats.sdkVersion = sdkVersion;
        stats.sdkId = SDK_ID;
        object.stats = stats;

        _this._ingestAsync(object);
      }

      return isEligible;
    }
  });
  this.apiKey = options.apiKey;
  this.envKey = options.envKey;
  this.timeout = 10 * 1000;
  this.pollingInterval = 60 * 1000;
  this.ingestionInterval = 60 * 1000;
  this.pollingThread = null;
  this.ingestionThread = null;
  this.gatingInfo = null;
  this.gatingInfoMap = null;
  this.ingestionBatch = [];
  this.ingestionMaxBatchSize = 500;
  var ajv = (0, _ajv.default)();
  this.validate = ajv.compile(SCHEMA);
  this.firstIngestion = true;
};

exports = module.exports = Airship;

